@@ -41,9 +41,6 @@ mod parser;
 mod relay;
 mod router;
 
-#[cfg(test)]
-mod tests;
-
 pub use cancel::*;
 pub use context::*;
 pub use executor::*;
