@@ -1,8 +1,4 @@
-use crate::relay::{RelayMessage, WorkerRelay};
-use crate::tokio::runtime::Runtime;
-use crate::tokio::sync::mpsc::{channel, Sender};
-use crate::{Context, Executor, Mailbox, NodeMessage};
-use ockam_core::compat::sync::Arc;
+use crate::{Context, Executor};
 use ockam_core::Address;
 #[cfg(feature = "std")]
 use tracing_subscriber::{filter::LevelFilter, fmt, EnvFilter};
@@ -10,21 +6,6 @@ use tracing_subscriber::{filter::LevelFilter, fmt, EnvFilter};
 /// A minimal worker implementation that does nothing
 pub struct NullWorker;
 
-impl NullWorker {
-    /// Create and register a new NullWorker context
-    pub fn create(
-        rt: Arc<Runtime>,
-        addr: &Address,
-        tx: Sender<NodeMessage>,
-    ) -> (Context, Sender<RelayMessage>) {
-        // Create a new Mailbox and Context
-        let (mb_tx, mb_rx) = channel(32);
-        let mb = Mailbox::new(mb_rx);
-
-        (Context::new(rt, tx, addr.into(), mb), mb_tx)
-    }
-}
-
 impl ockam_core::Worker for NullWorker {
     type Context = Context;
     type Message = (); // This message type is never used
@@ -37,14 +18,11 @@ pub fn start_node() -> (Context, Executor) {
     info!("Initializing ockam node");
 
     let mut exe = Executor::new();
-    let addr = "app".into();
+    let addr: Address = "app".into();
 
     // The root application worker needs a mailbox and relay to accept
     // messages from workers, and to buffer incoming transcoded data.
-    let (ctx, mb_tx) = NullWorker::create(exe.runtime(), &addr, exe.sender());
-
-    // Build a mailbox worker to buffer messages
-    let sender = WorkerRelay::<NullWorker, _>::build_root(&exe.runtime(), mb_tx);
+    let (ctx, sender, _) = Context::new(exe.runtime(), exe.sender(), addr.into());
 
     // Register this mailbox handle with the executor
     exe.initialize_system("app", sender);
